@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2010 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -970,10 +970,11 @@ static int AnnouncePunctuation(Translator *tr, int c1, int *c2_ptr, char *output
 #define SSML_BREAK    13
 #define SSML_IGNORE_TEXT 14
 #define HTML_BREAK    15
-#define SSML_CLOSE    0x10   // for a closing tag, OR this with the tag type
+#define HTML_NOSPACE  16    // don't insert a space for this element, so it doesn't break a word
+#define SSML_CLOSE    0x20   // for a closing tag, OR this with the tag type
 
 // these tags have no effect if they are self-closing, eg. <voice />
-static char ignore_if_self_closing[] = {0,1,1,1,1,0,0,0,0,1,1,0,1,0,1,0,0};
+static char ignore_if_self_closing[] = {0,1,1,1,1,0,0,0,0,1,1,0,1,0,1,0,0,0,0};
 
 
 static MNEM_TAB ssmltags[] = {
@@ -994,6 +995,7 @@ static MNEM_TAB ssmltags[] = {
 
 	{"br", HTML_BREAK},
 	{"li", HTML_BREAK},
+	{"dd", HTML_BREAK},
 	{"img", HTML_BREAK},
 	{"td", HTML_BREAK},
 	{"h1", SSML_PARAGRAPH},
@@ -1003,6 +1005,12 @@ static MNEM_TAB ssmltags[] = {
 	{"hr", SSML_PARAGRAPH},
 	{"script", SSML_IGNORE_TEXT},
 	{"style", SSML_IGNORE_TEXT},
+	{"font", HTML_NOSPACE},
+	{"b", HTML_NOSPACE},
+	{"i", HTML_NOSPACE},
+	{"strong", HTML_NOSPACE},
+	{"em", HTML_NOSPACE},
+	{"code", HTML_NOSPACE},
 	{NULL,0}};
 
 
@@ -1543,12 +1551,12 @@ static int ReplaceKeyName(char *outbuf, int index, int &outix)
 	int letter;
 	char *p;
 
-	p = &outbuf[index+1];
+	p = &outbuf[index];
 
 	if((letter = LookupMnem(keynames, p)) != 0)
 	{
 		ix = utf8_out(letter, p);
-		outix = index + ix + 1;
+		outix = index + ix;
 		return(letter);
 	}
 	return(0);
@@ -1636,16 +1644,26 @@ static int ProcessSsmlTag(wchar_t *xml_buf, char *outbuf, int &outix, int n_outb
 	
 	if(tag_name[0] == '/')
 	{
-		tag_type = LookupMnem(ssmltags,&tag_name[1]) + SSML_CLOSE;  // closing tag
+		// closing tag
+		if((tag_type = LookupMnem(ssmltags,&tag_name[1])) != HTML_NOSPACE)
+		{
+			outbuf[outix++] = ' ';
+		}
+		tag_type += SSML_CLOSE;
 	}
 	else
 	{
-		tag_type = LookupMnem(ssmltags,tag_name);
+		if((tag_type = LookupMnem(ssmltags,tag_name)) != HTML_NOSPACE)
+		{
+			// separate SSML tags from the previous word (but not HMTL tags such as <b> <font> which can occur inside a word)
+			outbuf[outix++] = ' ';
+		}
 
 		if(self_closing && ignore_if_self_closing[tag_type])
 			return(0);
 	}
 
+
 	voice_change_flag = 0;
 	terminator = CLAUSE_NONE;
 	ssml_sp = &ssml_stack[n_ssml_stack-1];
@@ -2178,9 +2196,7 @@ f_input = f_in;  // for GetC etc
 					}
 					xml_buf[n_xml_buf] = 0;
 					c2 = ' ';
-		
-					buf[ix++] = ' ';
-		
+	
 					self_closing = 0;
 					if(xml_buf[n_xml_buf-1] == '/')
 					{
@@ -2211,6 +2227,7 @@ f_input = f_in;  // for GetC etc
 						}
 						return(terminator);
 					}
+					c2 = GetC();
 					continue;
 				}
 			}
@@ -2510,6 +2527,13 @@ if(option_ssml) parag=1;
 
 				if(nl_count==0)
 				{
+					if((c1 == ',') && (cprev == '.') && (tr->translator_name == L('h','u')) && iswdigit(cprev2) && (iswdigit(c_next) || (iswlower(c_next))))
+					{
+						// lang=hu, fix for ordinal numbers, eg:  "december 2., szerda", ignore ',' after ordinal number
+						c1 = CHAR_COMMA_BREAK;
+						is_end_clause = 0;
+					}
+
 					if(c1 == '.')
 					{
 						if((tr->langopts.numbers & NUM_ORDINAL_DOT) && 
@@ -2522,7 +2546,7 @@ if(option_ssml) parag=1;
 							}
 							else
 							{
-								if (iswlower(c_next))
+								if (iswlower(c_next) || (c_next=='-'))     // hyphen is needed for lang-hu (eg. 2.-kal)
 									is_end_clause = 0;      // only if followed by lower-case, (or if there is a XML tag)
 							}
 						}
@@ -2565,7 +2589,7 @@ if(option_ssml) parag=1;
 					buf[ix] = ' ';
 					buf[ix+1] = 0;
 
-					if(!IsAlpha(c_next))
+					if(iswdigit(cprev) && !IsAlpha(c_next))   // ????
 					{
 						punct_data &= ~CLAUSE_DOT;
 					}
